/// IMPORTING THIRD PARTY PACKAGES
import 'package:flutter/material.dart';

/// IMPORTING LOCAL COMPONENTS
import 'package:zwap_design_system/atoms/atoms.dart';

import 'package:zwap_design_system/molecules/tag/zwapTag.dart';

/// The multi select form field
class ZwapMultiSelectFormFieldItem {

  /// The text to display on the list element
  String label;

  /// a boolean determining weather the element is selected or not
  bool isSelected;

  /// The value of the item, intended to be exploited later
  dynamic value;

  /// The style to apply to the label of the list
  final ZwapTextType labelStyle;

  /// The leading component inside the dropdown list element
  final Widget? leading;

  /// The trailing component inside the dropdown list element
  final Widget? trailing;

  ZwapMultiSelectFormFieldItem({
    required this.label,
    required this.value,
    this.isSelected = false,
    this.labelStyle = ZwapTextType.body1Regular,
    this.leading,
    this.trailing,
  });
}

/// Custom input with select tag
class ZwapMultiSelectFormField extends StatefulWidget {

  /// The list of elements
  final List<ZwapMultiSelectFormFieldItem> elementList;

  /// The label name for this input
  final String labelName;

  /// Is it an enabled input
  final bool isEnabled;

  ZwapMultiSelectFormField({Key? key,
    required this.elementList,
    required this.labelName,
    required this.isEnabled,
  }) : super(key: key);

  @override
  ZwapMultiSelectFormFieldState createState() => ZwapMultiSelectFormFieldState();
}

/// The state for this component
class ZwapMultiSelectFormFieldState extends State<ZwapMultiSelectFormField> {

  /// Retrieve the list of selected elements
  get selectedElements => widget.elementList.where((e) => e.isSelected == true).toList();

  /// Retrieve the list of unselected elements
  get unselectedElements => widget.elementList.where((e) => e.isSelected == false).toList();

  /// Is the dropdown open or not fro th
  bool isDropdownOpen = false;

  /// It opens or close the dropdown
  void _openCloseDropDown(){
    setState(() {
      this.isDropdownOpen = !this.isDropdownOpen;
    });
  }

  Widget _dropDownCard(Size size) => Container(
    height: size.height * 0.3,
    width: size.width,
    decoration: BoxDecoration(
      color: Colors.white,
      borderRadius: BorderRadius.circular(6.0),
      boxShadow: [
        BoxShadow(
          blurRadius: 4.0,
          color: Colors.black.withOpacity(0.15),
        ),
      ],
    ),
    child: Padding(
      padding: EdgeInsets.only(top: 55.0, left: 12.0, right: 12.0),
      child: ListView.builder(
        itemCount: widget.elementList.length,
        itemBuilder: (BuildContext context, int index) {
          final element = widget.elementList[index];
          return Material(
            color: Colors.transparent,
            child: InkWell(
              onTap: () {
                setState(() {
                  widget.elementList[index].isSelected = true;
                });
              },
              child: Padding(
                padding: const EdgeInsets.all(8.0),
                child: Row(
                  children: <Widget>[
                    if (element.leading != null) ...[
                      element.leading!,
                      SizedBox(width: 5)
                    ],
                    Expanded(
                      child: ZwapText(
                        zwapTextType: ZwapTextType.body1Regular,
                        textColor: ZwapColors.neutral700,
                        text: element.label,
                      ),
                    ),
                    if (element.trailing != null) ...[
                      SizedBox(width: 5),
                      element.trailing!,
                    ],
                  ],
                ),
              ),
            ),
          );
        },
      ),
    ),
  );

  @override
  Widget build(BuildContext context) {
    final Size size = MediaQuery.of(context).size;
    return Column(
      children: [
        Padding(
          padding: EdgeInsets.only(bottom: 3),
          child: ZwapText(
            zwapTextType: ZwapTextType.body1Regular,
            textColor: ZwapColors.neutral700,
            text: widget.labelName,
          ),
        ),
        Padding(
          padding: EdgeInsets.only(top: 3),
          child: Container(
            width: size.width,
            child: Stack(
              children: <Widget>[
                this.isDropdownOpen ? this._dropDownCard(size) : Container(),
                InkWell(
                  onTap: () => this._openCloseDropDown(),
                  child: _buildSelectedList(),
                )
              ],
            ),
          ),
        )
      ],
    );
  }

  /// It builds the selected elements list
  _buildSelectedList() {
    final Size size = MediaQuery.of(context).size;
    return Container(
      width: size.width,
      decoration: BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.circular(4.0),
        boxShadow: [
          BoxShadow(
            blurRadius: 2.0,
            color: Colors.black.withOpacity(0.1),
          ),
        ],
      ),
      child: Padding(
        padding: EdgeInsets.all(4.0),
        child: SingleChildScrollView(
          scrollDirection: Axis.horizontal,
          child: Row(
            children: <Widget>[
              ..._buildSelectedElementList(),
            ],
          ),
        ),
      ),
    );
  }

  /// It builds the component per each component inside the list
  _buildSelectedElementList() {
    final elementList =
    widget.elementList.where((e) => e.isSelected == true).toList();
    if (elementList.isEmpty) return [_buildEmptyLabel()];
    return elementList
        .map(
          (e) => ZwapTag(
        tagText: e.label,
        onCallBackClick: () {
          widget.elementList.where((elt) => elt == e).first.isSelected =
          false;
          setState(() {});
        },
      ),
    ).toList();
  }

  /// It builds the label for the empty state
  _buildEmptyLabel() {
    return Container(
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Container(),
      ),
    );
  }
}